/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo;

import java.awt.BorderLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeListener;

import com.teamdev.jxbrowser.WebBrowser;
import com.teamdev.jxbrowser.event.HistoryChangeEvent;
import com.teamdev.jxbrowser.event.HistoryChangeListener;
import com.teamdev.jxbrowser.event.LocationEvent;
import com.teamdev.jxbrowser.event.ProgressEvent;
import com.teamdev.jxbrowser.event.RequestAdapter;
import com.teamdev.jxbrowser.event.StateEvent;
import com.teamdev.jxbrowser.event.TitleChangeEvent;
import com.teamdev.jxbrowser.event.TitleChangeListener;

/**
 * @author Victor Gubin
 */
public final class TabbedBrowsersPane extends JComponent
{

    private static final long serialVersionUID = -8849832729797566984L;

    public static final String _untitled = "Untitled";

    public static final int BACKWARD_ACTION = 0;
    public static final int FORWARD_ACTION = 1;
    public static final int STOP_ACTION = 2;

    private class BrowserListener extends RequestAdapter implements
            TitleChangeListener, HistoryChangeListener
    {
        private final TitledPane titlePane;
        private final WebBrowser browser;

        public BrowserListener(TitledPane titlePane, WebBrowser browser)
        {
            this.titlePane = titlePane;
            this.browser = browser;
        }

        @Override
        public void locationChanged(LocationEvent event)
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    progressBar.setVisible(true);
                }
            });
        }

        @Override
        public void progressChanged(final ProgressEvent event)
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    progressBar.setMaximum((int) event.getMaxTotalProgress());
                    progressBar.setValue((int) event.getCurTotalProgress());
                }
            });
        }

        @Override
        public void stateChanged(StateEvent event)
        {
            if (event.isLoadingDocument())
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        commandActions[STOP_ACTION].setEnabled(true);
                    }
                });
            }

            if (event.isLoadingCompleted())
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        progressBar.setVisible(false);
                        commandActions[STOP_ACTION].setEnabled(false);
                    }
                });
            }
        }

        public void titleChanged(final TitleChangeEvent event)
        {
            SwingUtilities.invokeLater(new Runnable()
            {

                public void run()
                {
                    if (event.getTitle() == null || event.getTitle().equals("")
                            || event.getTitle().equals(_untitled)) return;
                    titlePane.setTitle(event.getTitle());

                    StringBuilder buffer = new StringBuilder();
                    if (event.getTitle().length() < 25)
                    {
                        buffer.replace(0, 25, "                        ");
                        buffer.insert(0, event.getTitle());
                        buffer.setLength(25);
                    }
                    else
                    {
                        buffer.append(event.getTitle());
                        buffer.setLength(25);
                        buffer.append("...");
                    }

                    int index = tabsRelationalship.indexOf(browser);
                    tabbedPane.setTitleAt(index, buffer.toString());
                }

            });
        }

        public void historyChanged(final HistoryChangeEvent event)
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    commandActions[BACKWARD_ACTION].setEnabled(event
                            .isBackwardEnabled());
                    commandActions[FORWARD_ACTION].setEnabled(event
                            .isForwardEnabled());
                }
            });
        }

    }

    private final AbstractAction closeTabAction;
    private final JProgressBar progressBar;
    private final Action[] commandActions;
    private final JTabbedPane tabbedPane;
    private final List<WebBrowser> tabsRelationalship;

    public TabbedBrowsersPane(JProgressBar progressBar, JPopupMenu popupMenu,
            final AbstractAction closeTabAction, final Action[] commandActions)
    {
        this.progressBar = progressBar;
        this.commandActions = commandActions;
        this.closeTabAction = closeTabAction;

        tabbedPane = new JTabbedPane(JTabbedPane.BOTTOM,
                JTabbedPane.SCROLL_TAB_LAYOUT);
        tabbedPane.setComponentPopupMenu(popupMenu);
        tabbedPane.addMouseListener(new MouseAdapter()
        {
            @Override
            public void mouseClicked(MouseEvent e)
            {
                if (e.getButton() == MouseEvent.BUTTON2)
                {
                    closeTabAction.actionPerformed(null);
                }
            }
        });
        tabsRelationalship = Collections
                .synchronizedList(new ArrayList<WebBrowser>());
        setLayout(new BorderLayout());
        add(tabbedPane, BorderLayout.CENTER);
    }

    public void addBrowser(final WebBrowser browser)
    {
        tabsRelationalship.add(browser);

        JPanel browserPanel = new JPanel(new BorderLayout());
        TitledPane titlePane = new TitledPane();
        titlePane.assignCloseAction(closeTabAction);
        browserPanel.add(titlePane, BorderLayout.NORTH);

        browserPanel.add(browser.getComponent(), BorderLayout.CENTER);
        tabbedPane.add(_untitled, browserPanel);
        tabbedPane.setSelectedComponent(browserPanel);
        

        BrowserListener listener = new BrowserListener(titlePane, browser);
        browser.addRequestListener(listener);
        browser.addTitleChangeListener(listener);
        browser.addHistoryChangeListener(listener);
    }

    public void removeBrowser()
    {
        if (tabbedPane.getTabCount() > 1)
        {
            WebBrowser browser = tabsRelationalship.remove(tabbedPane.getSelectedIndex());
            browser.dispose();
            tabbedPane.remove(tabbedPane.getSelectedIndex());
        }
    }

    public void removeBrowser(WebBrowser browser)
    {
        if (tabbedPane.getTabCount() > 1)
        {
            int index = tabsRelationalship.indexOf(browser);
            tabbedPane.remove(index);
            tabsRelationalship.remove(index);
        }
    }

    public int getTabCount()
    {
        return tabbedPane.getTabCount();
    }

    public void setTitle(String title)
    {
        tabbedPane.setTitleAt(tabbedPane.getSelectedIndex(), title);
    }

    public WebBrowser getActiveBrowser()
    {
        return tabsRelationalship.get(tabbedPane.getSelectedIndex());
    }

    public void addChangeListener(ChangeListener listener)
    {
        tabbedPane.addChangeListener(listener);
    }

/*    public void putBrowser(WebBrowser browser)
    {
        JComponent component = (JComponent) tabbedPane.getSelectedComponent();
        component.add(browser.getComponent(), BorderLayout.CENTER);
    }
*/
    public void disposeBrowsers() throws InterruptedException
    {
        for (final WebBrowser webBrowser : tabsRelationalship)
        {
            webBrowser.dispose();
        }
    }
}
